/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.camel.component.sqlplus

import org.apache.camel.Processor
import org.apache.camel.Exchange
import java.sql.ResultSet
import metridoc.camel.impl.iterator.ResultSetIteratorCreator
import metridoc.camel.batch.BatchIterator
import org.slf4j.LoggerFactory
import metridoc.plugins.sql.SqlPlus

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 8/5/11
 * Time: 9:46 AM
 */

class SqlPlusGProcessor extends SqlPlusGMixin implements Processor {

    SqlPlusGEndpoint endpoint
    static final log = LoggerFactory.getLogger(SqlPlusGProcessor)

    void process(Exchange exchange) {
        def isResultSet = exchange.in.getBody(ResultSet.class)
        def isMap = exchange.in.getBody(Map.class)
        def isList = exchange.in.getBody(List.class)
        def tableOrInsert = query

        if(isList) {
            handleListOrMap(tableOrInsert, isList)
        }

        if(isMap) {
            handleListOrMap(tableOrInsert, isMap)
        }

        if(isResultSet) {
            handleResultSet(tableOrInsert, isResultSet)
        }
    }

    private void handleListOrMap(String tableOrInsert, Object records) {
        try {
            sql.runBatch(tableOrInsert, records, logBatches)
        } finally {
            sql.close()
        }
    }

    private void handleResultSet(String tableOrInsert, ResultSet resultSet) {
        def resultSetIterator = new ResultSetIteratorCreator().create(resultSet)
        def batchIterator = new BatchIterator(batchSize: getBatchSize(), iterator: resultSetIterator)
        def sql = getSql()

        try {
            while (batchIterator.hasNext()) {
                def next = batchIterator.next()
                sql.runBatch(tableOrInsert, next, logBatches)
            }
        } finally {
            sql.close()
        }
    }
}
